// Header file for the testing bench 
// for the signaling less-than-or-equal comparison
// between a binary64 and a decimal64
// 
// Copyright (C) 2012, 2013 by
// 
// Laboratoire de l'Informatique du Parallelisme,
// UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
// 
// and by
// 
// Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
// UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France.
// 
// Authors : Christoph Lauter, Marc Mezzarobba
// 
// Contact : christoph.lauter@lip6.fr, marc@mezzarobba.net
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
// 

#ifndef COMPARISON_HELPER_H
#define COMPARISON_HELPER_H

#include <stdio.h>
#include <stdint.h>
#ifndef __STDC_WANT_DEC_FP__
#define __STDC_WANT_DEC_FP__ 1
#endif
#include <math.h>

typedef double _binary64;

#define BINARY64_CLASS_QNAN          0
#define BINARY64_CLASS_NEG_INF       1
#define BINARY64_CLASS_NEG_NORMAL    2
#define BINARY64_CLASS_NEG_SUBNORMAL 3
#define BINARY64_CLASS_NEG_ZERO      4
#define BINARY64_CLASS_POS_ZERO      5
#define BINARY64_CLASS_POS_SUBNORMAL 6
#define BINARY64_CLASS_POS_NORMAL    7
#define BINARY64_CLASS_POS_INF       8
#define BINARY64_CLASS_SNAN          9

#define BINARY64_NUMBER_CLASSES     10


#define DECIMAL64_CLASS_QNAN         0
#define DECIMAL64_CLASS_NEG_INF      1
#define DECIMAL64_CLASS_NEG_NUMBER   2
#define DECIMAL64_CLASS_NEG_ZERO     3
#define DECIMAL64_CLASS_POS_ZERO     4
#define DECIMAL64_CLASS_POS_NUMBER   5
#define DECIMAL64_CLASS_POS_INF      6
#define DECIMAL64_CLASS_SNAN         7

#define DECIMAL64_NUMBER_CLASSES     8


#define PAIR_HARD 0
#define PAIR_EASY 1
#define PAIR_OPPSIGNS 2
#define PAIR_SPECIAL 3

_binary64 composeBinary(int negative, int E, uint64_t m);
_Decimal64 composeDecimal(int negative, int expo, uint64_t mant);

int classifyBinary(_binary64 x);
int classifyDecimal(_Decimal64 x);

int classifyCase(_binary64 x, _Decimal64 y);

char *binaryClassToName(int class);
char *decimalClassToName(int class);


#endif /* ifdef COMPARISON_HELPER_H*/
